#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include "ast.hh"
#include "parser.hh"
#include "symtab.hh"
#include <unistd.h>

extern int yyparse(void);
extern int yydebug;
extern int errorCount;
extern int warningCount;

static const char *OPTION_STRING = "dhc";

void Usage(char *program)
{
    cerr << "Usage:\n"
         << program << " [-d|-c] [filename]\n"
         << program << " -h\n"
         << "\n"
         << "Options:\n"
         << "  -h               Shows this message.\n"
         << "  -d               Turn on parser debugging.\n"
         << "  -c               Generates a C-representation of the program\n";

    exit(1);
}

int main(int argc, char **argv)
{
    int          option;
    extern FILE *yyin;

    /*
     Set up the symbol table.
     Adds builtin functions used by the language.
     Builtin functions include routines to print reals integers etc.
    */

    currentFunction = new FunctionInformation("main");
    kIntegerType    = new TypeInformation("integer", sizeof(long));
    kRealType       = new TypeInformation("real", sizeof(double));

    kFPrintFunction = new FunctionInformation("putreal");
    FunctionInformation *kPLReadFunction = new FunctionInformation("putline");
    kIPrintFunction = new FunctionInformation("putint");
    kFReadFunction = new FunctionInformation("getreal");
    kIReadFunction = new FunctionInformation("getint");

    kIPrintFunction->SetReturnType(kIntegerType);
    kIPrintFunction->AddParameter("x", kIntegerType);
    kFPrintFunction->SetReturnType(kIntegerType);
    kFPrintFunction->AddParameter("x", kRealType);
    kIReadFunction->SetReturnType(kIntegerType);
    kFReadFunction->SetReturnType(kRealType);
    kIReadFunction->SetReturnType(kIntegerType);

    currentFunction->AddSymbol(kIntegerType);
    currentFunction->AddSymbol(kRealType);
    currentFunction->AddSymbol(kIPrintFunction);
    currentFunction->AddSymbol(kFPrintFunction);
    currentFunction->AddSymbol(kIReadFunction);
    currentFunction->AddSymbol(kFReadFunction);
    currentFunction->AddSymbol(kPLReadFunction);

    //
    // Check command-line arguments
    //

    opterr = 0;
    optopt = '?';
    while ((option = getopt(argc, argv, OPTION_STRING)) != EOF)
    {
        switch (option)
        {
        case 'd':
            yydebug = 1;
            break;
        case 'h':
            Usage(argv[0]);
            break;
        case '?':
            Usage(argv[0]);
            break;
        case 'c':
          std::cout << "/*Using standard-library*/\n";
          std::cout << "#include \"stl.h\"\n";
          SymbolInformation::outputFormat = SymbolInformation::kCFormat;
          currentFunction->outputFormat = SymbolInformation::kCFormat;
          break;
        }
    }

    if (argv[optind])
    {
        yyin = fopen(argv[optind], "r");
        if (yyin == NULL)
        {
            perror(argv[optind]);
            exit(1);
        }
        optind += 1;
    }

    if (optind < argc)
        Usage(argv[0]);

    //
    // Compile the input
    //

    int retcode = yyparse();
    return retcode;
}
